import random
import time
import curses
from termcolor import colored
from colorama import init, Fore, Style
import os

# Initialize colorama
init()

# File to store the high score
HIGH_SCORE_FILE = "high_score.txt"

# Define fish types
fish_types = [
    "Red Snapperdoodle", "Blue Whistlefish", "Green Gobbler", "Yellow Sunfin",
    "Purple Wigglefish", "Orange Specklefin", "Pink Jellywhisker", "Brown Muckfish",
    "Black Pebblegill", "White Cloudfin", "Striped Hummingfish", "Spotted Glowfish",
    "Golden Scalegrin", "Silver Dartfish", "Copper Beakfish", "Bronze Leafswimmer",
    "Rainbow Swirlfish", "Dotted Spinfish", "Swirling Mistfish", "Leaping Froggill",
    "Glowing Stardancer", "Shimmering Moonfish", "Twinkling Starlightfish",
    "Flaming Emberfish", "Chilly Frostfish", "Breezy Windfin", "Stormy Thunderfish",
    "Calm Ripplefin", "Gentle Wavefish", "Splashing Watergill", "Deep Sea Shadowfish",
    "Mystic Dreamfish", "Enchanted Fairfish", "Magic Charmfish", "Cursed Spookfish",
    "Ethereal Ghostfin", "Ancient Fossilfish", "Whispering Nightfish", "Luminous Dayfish",
    "Enigmatic Maze Fish", "Mysterious Phantomfish", "Silent Mistyfish", "Singing Songfish",
    "Pulsating Heartfish", "Rapid Racerfish", "Gliding Sailfish", "Darting Arrowfish",
    "Serene Peacefish", "Whirling Dervishfish"
]

# Define random events
random_events = [
    "A snail tips its hat to you as it trundles by.",
    "A butterfly flutters past your nose.",
    "A frog jumps onto a lily pad nearby.",
    "A fish jumps out of the water and winks at you.",
    "A rabbit hops by, carrying a pocket watch.",
    "A bird lands on your fishing rod and chirps.",
    "A squirrel chatters at you from a tree.",
    "A deer drinks from the edge of the pond.",
    "A turtle slowly makes its way to the water.",
    "A dragonfly buzzes around your head.",
    "A friendly bee hovers by your ear.",
    "A small mouse scurries past your feet.",
    "A pair of ducks swim by quacking happily.",
    "A raccoon washes its paws in the water.",
    "A fox peeks at you from behind a bush.",
    "A hedgehog snuffles around the grass.",
    "A butterfly lands on your shoulder.",
    "A ladybug crawls onto your fishing hat.",
    "A fish splashes you with water playfully.",
    "A frog croaks a cheerful tune.",
    "A rabbit nibbles on some grass nearby.",
    "A bird sings a melodious song.",
    "A deer looks at you curiously.",
    "A squirrel runs up a tree trunk.",
    "A dragonfly rests on a nearby rock.",
    "A snail leaves a shiny trail behind.",
    "A fish swims in circles in the water.",
    "A rabbit wiggles its nose at you.",
    "A frog jumps and catches a fly.",
    "A bird builds a nest in a tree.",
    "A squirrel finds an acorn and hides it.",
    "A butterfly dances in the air.",
    "A turtle stretches its neck to look at you.",
    "A fox yawns and lies down.",
    "A raccoon digs for food in the soil.",
    "A fish pops its head out of the water.",
    "A frog leaps into the pond with a splash.",
    "A rabbit scratches its ear with a foot.",
    "A bird preens its feathers on a branch.",
    "A deer grazes quietly nearby.",
    "A squirrel chatters happily in the trees.",
    "A dragonfly zips past your nose.",
    "A snail slowly climbs a blade of grass.",
    "A fish swims close to the shore.",
    "A rabbit stretches and yawns.",
    "A bird hops along the ground.",
    "A squirrel digs a hole in the ground.",
    "A butterfly flits from flower to flower.",
    "A turtle basks in the sunlight."
]

# Define weather events
weather_events = [
    "It's a sunny day.",
    "It's raining.",
    "It's snowing.",
    "It's windy.",
    "It's cloudy.",
    "It's a stormy day.",
    "It's foggy.",
    "It's a clear night with stars.",
    "It's drizzling.",
    "There's a rainbow in the sky."
]

# Define magic rods with reduced multipliers
magic_rods = [
    ("Enchanted Willow Rod", 0.06), ("Mystic Moon Rod", 0.12), ("Fairy Dust Rod", 0.18),
    ("Wizard's Wand Rod", 0.24), ("Dragon Scale Rod", 0.30), ("Phoenix Feather Rod", 0.36),
    ("Mermaid's Tail Rod", 0.42), ("Giant's Strength Rod", 0.48), ("Elven Silver Rod", 0.54),
    ("Celestial Star Rod", 0.60)
]

# Generate fish with weights and points
fish_data = []
for fish in fish_types:
    weight = round(random.uniform(0.1, 10.0), 1)
    points = int(weight * 100) + random.randint(1, 200)
    fish_data.append((fish, weight, points))

# Scenes
scenes = {
    "sunny": [
        "You see a bright, sunny sky above the strange woods of Wonderland. The trees have colorful leaves that sway gently in the breeze.",
        "The sun casts a golden glow over the whimsical landscape. A peculiar bird sings an enchanting melody.",
        "Sunlight filters through the tall, oddly-shaped trees, creating a dappled pattern on the ground.",
        "A curious rabbit with a pocket watch hops by, glancing at the vibrant flowers blooming nearby.",
        "A serene lake reflects the clear blue sky, with an odd sunrise adding a surreal touch to the scene."
    ],
    "raining": [
        "Raindrops fall gently, creating ripples in the lake. The trees glisten as the rain enhances their vibrant colors.",
        "You see a misty rain falling over the woods, giving the surroundings a mystical, dream-like quality.",
        "The sound of rain is soothing as it taps on the leaves and the surface of the lake.",
        "A frog hops onto a lily pad, enjoying the gentle rain shower.",
        "The rain brings out the earthy smell of the forest, and a rainbow appears in the distance."
    ],
    "snowing": [
        "Snowflakes drift down, covering the landscape in a blanket of white. The trees look magical with their frosty branches.",
        "You see a winter wonderland, with snow gently falling and a serene quietness all around.",
        "The lake is partially frozen, and snowflakes dance in the air, adding to the enchantment of the scene.",
        "A snow-covered rabbit hops by, leaving tiny footprints in the snow.",
        "Icicles hang from the trees, sparkling in the light, and a snowman stands cheerfully near the lake."
    ],
    "windy": [
        "The wind rustles the leaves of the trees, creating a soothing symphony of natural sounds.",
        "You see the trees swaying in the wind, their colorful leaves fluttering like butterflies.",
        "The wind creates waves on the lake, making it look like a dancing mirror.",
        "A kite shaped like a dragon flies high in the sky, carried by the strong wind.",
        "The breeze carries the scent of blooming flowers and fresh pine, adding to the surreal atmosphere."
    ],
    "cloudy": [
        "Gray clouds drift across the sky, casting soft shadows on the whimsical landscape.",
        "You see the trees standing tall and still, with the lake reflecting the cloudy sky.",
        "A sense of calm pervades the air as the clouds move slowly overhead.",
        "A group of ducks swims peacefully on the lake, undisturbed by the cloudy weather.",
        "The muted light makes the colors of the trees and flowers appear even more vivid and enchanting."
    ],
    "foggy": [
        "A thick fog blankets the landscape, giving it an eerie yet magical feel.",
        "The fog swirls around the trees, creating ghostly shapes that vanish as quickly as they appear.",
        "Visibility is low, but you can just make out the shapes of strange creatures moving in the mist.",
        "The fog dampens all sounds, making the world feel eerily silent and still.",
        "You see a shadowy figure in the distance, but it's gone when you blink."
    ],
    "stormy": [
        "Dark clouds swirl overhead as a storm brews, adding a sense of drama to the landscape.",
        "Lightning flashes in the distance, illuminating the eerie shapes of the trees.",
        "The wind howls as the storm approaches, bending the trees and stirring the lake.",
        "A bolt of lightning strikes a tree, splitting it in half with a deafening crack.",
        "The storm's fury is both terrifying and awe-inspiring, casting the landscape in a surreal light."
    ],
    "clear": [
        "The night sky is clear, with stars twinkling like diamonds above the serene landscape.",
        "A full moon casts a silver glow over the whimsical trees and the calm lake.",
        "The stars form strange and beautiful constellations, adding to the magic of the night.",
        "A shooting star streaks across the sky, leaving a trail of light in its wake.",
        "The night air is cool and refreshing, filled with the sounds of nocturnal creatures."
    ],
    "drizzling": [
        "A light drizzle falls, creating a soft, rhythmic patter on the leaves and the lake.",
        "The drizzle adds a gentle, misty quality to the landscape, blurring the edges of reality.",
        "Tiny droplets cling to the flowers and leaves, making them sparkle in the muted light.",
        "A gentle breeze carries the fresh, earthy scent of rain through the air.",
        "The drizzle is soothing, creating a peaceful and calming atmosphere."
    ],
    "rainbow": [
        "A beautiful rainbow arches across the sky, adding a touch of magic to the landscape.",
        "The rainbow's vibrant colors stand out against the dark clouds, creating a stunning contrast.",
        "Raindrops glisten like jewels as the sun shines through the clouds, illuminating the rainbow.",
        "A double rainbow appears, doubling the beauty and enchantment of the scene.",
        "The rainbow seems to end at the lake, creating a magical and surreal vista."
    ]
}

# Game state
catch_list = []
total_score = 0
fishing = True
last_wiggle_time = 0
current_rod = ("Basic Rod", 0.0)
high_score = 0

def load_high_score():
    if os.path.exists(HIGH_SCORE_FILE):
        with open(HIGH_SCORE_FILE, "r") as file:
            return int(file.read().strip())
    return 0

def save_high_score(score):
    with open(HIGH_SCORE_FILE, "w") as file:
        file.write(str(score))

def display_inventory(pad, start_row, max_y):
    pad.addstr(start_row, 0, "\nYour catch list:\n", curses.color_pair(3))
    start_row += 1
    for fish in catch_list:
        pad.addstr(start_row, 0, f"Type: {fish[0]}, Weight: {fish[1]} lbs, Points: {fish[2]}\n", curses.color_pair(3))
        start_row += 1
    pad.addstr(start_row, 0, f"\nCurrent Rod: {current_rod[0]} (Catch multiplier: {current_rod[1] * 100:.1f}%)\n\n", curses.color_pair(3))  # Added line break after current rod
    start_row += 3
    pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, curses.COLS - 1)
    return start_row

def fish_event(pad, start_row, max_y):
    global total_score
    fish = random.choice(fish_data)
    catch_list.append(fish)
    total_score += fish[2]
    pad.addstr(start_row, 0, f"You caught a {fish[0]} weighing {fish[1]} lbs worth {fish[2]} points!\n", curses.color_pair(4))
    start_row += 1
    pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, curses.COLS - 1)
    return start_row

def wiggle_rod(pad, start_row, max_y):
    global last_wiggle_time
    if time.time() - last_wiggle_time < 60:
        pad.addstr(start_row, 0, "No more wiggles left, try again soon!\n", curses.color_pair(5))
        start_row += 1
    else:
        last_wiggle_time = time.time()
        start_row = fish_event(pad, start_row, max_y)
    pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, curses.COLS - 1)
    return start_row

def look_around(pad, start_row, max_y, weather):
    weather_key = weather.split()[2].lower().replace('.', '') if "clear" in weather or "rainbow" in weather else weather.split()[1].lower().replace('.', '')
    scene = random.choice(scenes.get(weather_key, scenes["sunny"]))
    pad.addstr(start_row, 0, f"\n{scene}\n", curses.color_pair(3))
    start_row += 2
    pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, curses.COLS - 1)
    return start_row

def catch_new_rod(pad, start_row, max_y):
    global current_rod
    new_rod = random.choice(magic_rods)
    current_rod = new_rod
    pad.addstr(start_row, 0, f"\nYou caught a new rod: {new_rod[0]}! (Catch multiplier: {new_rod[1] * 100:.1f}%)\n", curses.color_pair(3))
    start_row += 2
    pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, curses.COLS - 1)
    return start_row

def fishing_sim(stdscr):
    global fishing
    global current_rod
    global total_score
    global high_score

    curses.start_color()
    curses.init_pair(1, curses.COLOR_WHITE, curses.COLOR_BLACK)
    curses.init_pair(2, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.init_pair(3, curses.COLOR_YELLOW, curses.COLOR_BLACK)
    curses.init_pair(4, curses.COLOR_GREEN, curses.COLOR_BLACK)
    curses.init_pair(5, curses.COLOR_MAGENTA, curses.COLOR_BLACK)

    max_y, max_x = stdscr.getmaxyx()
    pad = curses.newpad(1000, max_x)
    pad.bkgd(' ', curses.color_pair(1))

    stdscr.addstr(1, 0, " W O N D E R L A N D   F I S H I N G   S I M\n", curses.color_pair(3))
    stdscr.addstr(3, 0, " Created by Tengushee Interactive...\n", curses.color_pair(4))
    stdscr.addstr(4, 0, " https://interactive.tengushee.com", curses.color_pair(3))
    stdscr.addstr(6, 0, " A shard of The Endless Chronicles...\n", curses.color_pair(4))
    stdscr.addstr(7, 0, " https://www.endless-chronicles.com", curses.color_pair(3))
    stdscr.addstr(9, 0, " Loading, please wait...", curses.color_pair(4))
    stdscr.refresh()
    time.sleep(2)  # Simulate loading time

    high_score = load_high_score()

    start_row = 0
    pad.addstr(start_row, 0, "\nWelcome to Wonderland Fishing Sim. You've started fishing!\n\n", curses.color_pair(1))  # Added line break after welcome message
    start_row += 3
    weather = random.choice(weather_events)
    pad.addstr(start_row, 0, f"Current weather: {weather}\n", curses.color_pair(2))
    start_row += 1
    pad.addstr(start_row, 0, f"High Score: {high_score}\n", curses.color_pair(2))
    start_row += 2
    pad.addstr(start_row, 0, "Press 'Q' to quit, 'I' to view your inventory, 'W' to wiggle your rod, and 'L' to look around.\n", curses.color_pair(1))
    start_row += 1
    pad.refresh(0, 0, 0, 0, max_y - 1, max_x - 1)

    next_fish_time = random.uniform(5, 30)
    last_message_time = time.time()
    last_weather_time = time.time()

    while fishing:
        current_time = time.time()

        if current_time - last_message_time >= 10:
            pad.addstr(start_row, 0, "You are fishing...\n", curses.color_pair(4))
            start_row += 1
            if random.random() < 0.03 + current_rod[1]:  # Adjusted catch chance based on rod multiplier
                start_row = fish_event(pad, start_row, max_y)
            last_message_time = current_time
            pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, max_x - 1)

        if current_time - last_weather_time >= 60:
            weather = random.choice(weather_events)
            pad.addstr(start_row, 0, f"\nCurrent weather: {weather}\n", curses.color_pair(2))
            start_row += 2
            last_weather_time = current_time
            pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, max_x - 1)

        if random.random() < 0.025 and current_time - last_message_time >= 5:  # Reduced random event frequency by 50%
            event = random.choice(random_events)
            pad.addstr(start_row, 0, event + "\n", curses.color_pair(1))
            start_row += 1
            last_message_time = current_time
            pad.refresh(start_row - max_y, 0, 0, 0, max_y - 1, max_x - 1)

        if random.random() < 0.001:  # 0.1% chance to catch a new rod
            start_row = catch_new_rod(pad, start_row, max_y)

        stdscr.nodelay(True)
        key_press = stdscr.getch()
        if key_press != -1:
            if chr(key_press).lower() == 'q':
                fishing = False
            elif chr(key_press).lower() == 'i':
                start_row = display_inventory(pad, start_row, max_y)
            elif chr(key_press).lower() == 'w':
                start_row = wiggle_rod(pad, start_row, max_y)
            elif chr(key_press).lower() == 'l':
                start_row = look_around(pad, start_row, max_y, weather)

        time.sleep(0.1)

    # Save high score if current score is higher
    if total_score > high_score:
        save_high_score(total_score)
        high_score = total_score

def main():
    curses.wrapper(fishing_sim)
    print()
    print(Fore.RED + " Wonderland Fishing Sim Final Score:")
    print()
    print(Fore.RED + f" Final Score: {total_score}")
    print(Fore.RED + f" High Score: {high_score}")
    print()
    print(Fore.RED + " If you enjoyed this game please donate to:")
    print()
    print(Fore.RED + " TEZ: Tengushee.tez")
    print(Fore.RED + " ETH: 0x934f5fb4fae01bae3fc26d155a21b037821e7e72")
    print(Fore.RED + " SOL: GhBsh2K9w1WUuypkBJVC8V5BerSvbyYgpTizirRXNBMd")
    print(Style.RESET_ALL)

if __name__ == "__main__":
    main()
